#include <stdio.h>
#include <stdbool.h>
#include <ctype.h>

// Size of each file read operation
#define READ_SIZE 10

// the user should pass in one argument: the file to print out
int main(int argc, char* argv[]) {

    // check that the user passed in an argument
    if (argc != 2) {
        fprintf(stderr, "Usage: %s FILE\n", argv[0]);
        return 1;
    }

    // try using the second argument as a filepath and open it
    FILE* infile = fopen(argv[1], "r");
    if (infile == NULL) {
        fprintf(stderr, "ERROR: failed to open file %s\n", argv[1]);
        return 2;
    }

    // create a local array to hold data
    char read_data[READ_SIZE];

    // Loop through data
    while (true) {
        // read from file
        size_t read_amount = fread(read_data, sizeof(char), READ_SIZE, infile);

        // break if end-of-file was reached
        if (read_amount == 0) {
            // we hit either end-of-file or an error
            if (feof(infile) != 0) {
                // end-of-file occurred
                break;
            } else if (ferror(infile) != 0) {
                // error occurred
                fprintf(stderr, "ERROR: error while reading file %s\n", argv[1]);
                return 3;
            } else {
                // something else???
                fprintf(stderr, "ERROR: unknown error while reading file %s\n", argv[1]);
                return 4;
            }
        }

        // print out data
        size_t write_amount = fwrite(read_data, sizeof(char), read_amount, stdout);
        if (write_amount < read_amount) {
            fprintf(stderr, "ERROR: could not write output\n");
            return 5;
        }
        /*
        for (size_t i=0; i<read_amount; i++) {
            printf("%c", read_data[i]);
        }
        */
    }

    // close file and exit successfully
    fclose(infile);
    return 0;
}

