#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>

#define STRING_BASE_SIZE 1

char* read_line(void) {

    // allocate space to hold string
    char* string = (char*)malloc(STRING_BASE_SIZE * sizeof(char));
    if (string == NULL) {
        fprintf(stderr, "ERROR: out of memory\n");
        exit(1);
    }
    size_t string_length = STRING_BASE_SIZE;

    // loop through stdin, reading characters
    size_t pos = 0;
    while (true) {
        // read a character from input
        int read_character = getchar();
        
        // stop looping if we could not read
        if (read_character == EOF && pos == 0) {
            // we did NOT read in some characters before end-of-file
            free(string);
            return NULL;
        } else if (read_character == EOF || read_character == '\n') {
            // some characters were read
            string[pos] = '\0';
            return string;
        }

        // copy character into output
        string[pos] = read_character;
        pos++;

        // check if we need to expand the string
        // make sure at least 1 character is always remaining (for null terminator)
        if ((string_length-pos) < 1) {

            // reallocate string
            size_t new_string_length = string_length + 1;
            char* new_string = (char*)realloc(string, new_string_length);
            if (new_string == NULL) {
                fprintf(stderr, "ERROR: out of memory\n");
                free(string);
                exit(1);
            }

            // overwrite existing string pointer with new one
            string = new_string;
            string_length = new_string_length;

            /*
            // THIS IS THE MALLOC VERSION INSTEAD OF REALLOC
            // allocate new bigger string
            size_t new_string_length = string_length + 1;
            char* new_string = (char*)malloc(new_string_length * sizeof(char));
            if (new_string == NULL) {
                fprintf(stderr, "ERROR: out of memory\n");
                exit(1);
            }

            // copy over contents
            for (size_t i=0; i<string_length; i++) {
                new_string[i] = string[i];
            }

            // replace original string
            free(string);
            string = new_string;
            string_length = new_string_length;
            */
        }
    }
}

int main(void) {

    while (true) {
        // read line from stdin
        char* line = read_line();

        // exit if input is complete
        if (line == NULL) {
            break;
        }

        // print out the line
        printf("LINE: %s\n", line);

        // free the line
        free(line);
    }

    return 0;
}

